#!/bin/sh
  Version=2.0.8
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      04/04/2003
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:   See ShowUsage().
#
# History: 1.00 - Created.
#          1.01 - Updated help.
#                 Adjusted output of -i option so that files beginning
#                 with a period are not shown.
#          1.02 - Supressed exit message in ExitScript function.
#          1.03 - Adjusted signature retrieval.
#          1.04 - Added -c option.
#                 Removed ShowVersion function.
#          1.05 - Modified for OS 10.5 compatibility.
#                 Adjusted for NAV 11.
#          1.06 - Improved how ScriptDir is defined.
#          1.07 - Modified for case-sensitive volume compatibility.
#          1.08 - Added display of NAV 12-style engine and hub folders.
#          1.09 - Added display of build number.
#                 Limited search to virscan files when looking for the
#                 first available date in a folder.
#          2.0.0 - 11/12/2013 - Corey Swertfager:
#                  * Added support for NIS 6.
#          2.0.1 - 01/16/2014 - Corey Swertfager:
#                  * Fixed how AntiVirusDir is assigned.
#          2.0.2 - 10/18/2016 - Corey Swertfager:
#                  * Added navlibx version (Etrack 4002770).
#          2.0.3 - 10/20/2016 - Corey Swertfager:
#                  * Updated how navlibx version is obtained for compatibility
#                    with older versions of otool.
#          2.0.4 - 06/24/2019 - Corey Swertfager:
#                  * Added support for multiple Silo directories.
#          2.0.5 - 08/17/2019 - Corey Swertfager:
#                  * Now only runs otool if its man pages are installed.
#          2.0.6 - 04/03/2020 - Corey Swertfager:
#                  * Changed Author company to NortonLifeLock Inc.
#          2.0.7 - 11/04/2020 - Corey Swertfager:
#                  * Added copyright section.
#          2.0.8 - 12/21/2020 - Corey Swertfager:
#                  * Updated copyright section.

# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
AbbreviatedScriptName="`basename "$0" .command`"
AntiVirusDirOld="/Library/Application Support/Norton Solutions Support/Norton AntiVirus"
AntiVirusDir2="/Library/Application Support/Symantec/AntiVirus"
AntiVirusDirs6=`ls -d "/Library/Application Support/Symantec/Silo/"*"/Definitions/virusdefs" 2>/dev/null`
FullScriptName="`basename "$0"`"
LANG=""
ScriptDir="`dirname "$0"`"
[ -z "`echo "z$ScriptDir" | grep '^z/'`" ] && ScriptDir="`/bin/pwd -P`/$ScriptDir"
ShowIndividualFiles=false

# *** Function Declarations ***

ExitScript()
{
   # Usage:     ExitScript [$1]
   # Argument:  $1 = The value to pass when calling exit. If nothing
   #                 is passed, then 0 is passed to exit.
   # Summary:   Checks to see if RunScriptAsStandAlone variable was
   #            set to true. If so, a message is displayed; otherwise,
   #            no message is displayed. The script is then exited and
   #            passes $1 to exit command.
   #
   if [ -z "$1" ] ; then
      exit 0
   else
      exit $1
   fi
}

GetDefsDate()
{
   # Usage:     GetDefsDate file
   # Summary:   Gets the date from file if it is a virus def file.
   #            Assigns date to DefsDate variable and returns 0 if
   #            a date is found, else assigns NULL to DefsDate and
   #            returns 1.
   #
   local FilePassed="$1"
   local DefsFolder
   local navlibxVersion
   DefsDate=""
   # If not a file, return 1
   [ -f "$FilePassed" ] || return 1
   # If $1 has a filename that begins with a period, return 1
   [ "`basename "/$FilePassed" | awk '{print substr($0,1,1)}'`" = "." ] && return 1
   Signature8=NotSetYet
   DefsBuild=N/A
   DefsMonth=""
   IFS='
'
   for Line in `hexdump -C -v -n 20 "$FilePassed"` ; do
      if [ "$Signature8" = NotSetYet ] ; then
         FirstLine="$Line"
         Signature4=`echo "z$Line" | awk '{print $2$3$4$5}'`
         Signature8=`echo "z$Line" | awk '{print $2$3$4$5$6$7$8$9}'`
      # Else if Signature4 = SYMC
      elif [ "z$Signature4" = z53594d43 ] ; then
         DefsMonth="`echo "z$FirstLine" | awk '{print $10}'`"
         DefsMonth="`printf %d 0x$DefsMonth`"
         DefsDay="`echo "z$FirstLine" | awk '{print $11}'`"
         DefsDay="`printf %d 0x$DefsDay`"
         DefsYear="`echo "z$FirstLine" | awk '{print $12}'`"
         DefsYear="`printf %d 0x$DefsYear`"
         DefsYear="`expr $DefsYear + 1900`"
         DefsBuild="`echo "z$FirstLine" | awk '{print $9$8$7$6}'`"
         DefsBuild="`printf "%d" 0x$DefsBuild`"
         break
      # Else if Signature8 = SYMPATCH
      elif [ "z$Signature8" = z53594d5041544348 ] ; then
         DefsMonth="`echo "z$Line" | awk '{print $2}'`"
         DefsMonth="`printf %d 0x$DefsMonth`"
         DefsDay="`echo "z$Line" | awk '{print $3}'`"
         DefsDay="`printf %d 0x$DefsDay`"
         DefsYear="`echo "z$Line" | awk '{print $5$4}'`"
         DefsYear="`printf %d 0x$DefsYear`"
         break
      else
         DefsMonth=""
         break
      fi
   done
   # If file type was not correct, return 1
   [ -z "$DefsMonth" ] && return 1
   case $DefsMonth in
      1) DefsMonthName=January
         ;;
      2) DefsMonthName=February
         ;;
      3) DefsMonthName=March
         ;;
      4) DefsMonthName=April
         ;;
      5) DefsMonthName=May
         ;;
      6) DefsMonthName=June
         ;;
      7) DefsMonthName=July
         ;;
      8) DefsMonthName=August
         ;;
      9) DefsMonthName=September
         ;;
      10) DefsMonthName=October
         ;;
      11) DefsMonthName=November
         ;;
      12) DefsMonthName=December
         ;;
   esac
   DefsFolder=`dirname "$FilePassed"`
   man otool &>/dev/null
   # If otool man pages are installed
   if [ $? = 0 ] ; then
      # Attempt to obtain navlibx version
      TempOtoolFile="/tmp/GetDatesDate-otool-`date +"%Y%m%d%H%M%S"`.plist"
      otool -s __TEXT __info_plist "$DefsFolder/navlibx" 2>/dev/null | xxd -r - "$TempOtoolFile" 2>/dev/null
      navlibxVersion=`cat -v "$TempOtoolFile" 2>/dev/null | grep CFBundleShortVersionString -A1 | tail -n 1 | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
      rm -f "$TempOtoolFile" 2>/dev/null
   else
      navlibxVersion=""
   fi
   DefsDate="$DefsMonthName $DefsDay, $DefsYear (build $DefsBuild)`[ "$navlibxVersion" ] && echo " (navlibx $navlibxVersion)"`"
   return 0
}

ShowDefsDateForFileOrFolder()
{
   # Usage:     ShowDefsDateForFileOrFolder $1
   # Argument:  $1 = Name of file or folder to check
   # Summary:   Shows virus defs date for file or folder passed.
   #
   CurrentDir="`pwd`"
   DefsDate=""
   if [ -d "$1" -a $ShowIndividualFiles = false ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls | grep -ie virscan` ; do
         GetDefsDate "$FileToCheck"
         # If DefsDate was assigned, break
         [ -n "$DefsDate" ] && break
      done
      if [ -n "$DefsDate" ] ; then
         echo "Date of virus defs in folder '$1':"
         echo "   $DefsDate"
      else
         echo "No virus defs date was found in any file within folder: $1"
      fi
      echo
   elif [ -d "$1" -a $ShowIndividualFiles = true ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetDefsDate "$FileToCheck"
         if [ -n "$DefsDate" ] ; then
            echo "Date of virus defs file '$1/$FileToCheck':"
            echo "   $DefsDate"
            echo
         else
            # If filename does not begin with a period (is not invisible)
            if [ "`basename "$FileToCheck" | awk '{print substr($0,1,1)}'`" != "." ] ; then
               echo "No virus defs date was found in file: $1/$FileToCheck"
               echo
            fi
         fi
      done
   elif [ -f "$1" ] ; then
      GetDefsDate "$1"
      if [ -n "$DefsDate" ] ; then
         echo "Date of virus defs file '$1':"
         echo "   $DefsDate"
      else
         echo "No virus defs date was found in file: $1"
      fi
      echo
   else
      echo "No such file or folder: '$1'"
      echo
   fi
   cd "$CurrentDir"
}

ShowUsage()
{
   # Usage:     ShowUsage
   # Summary:   Displays script usage message and exits script.
   #
   clear
   echo "
Usage:   $FullScriptName [-c] [-i] [files] [folders]

Summary: Shows virus defs dates and builds of Engine, Hub, NewEngine,
         SavedEngine and NAV 12-style virus definitions folders in the
         AntiVirus support folder if no files or folders are passed;
         otherwise shows virus defs dates of all files or folders passed.

         $FullScriptName will report the first virscan date it finds in a
         given folder unless the -i option is passed.

         If -c is passed, then screen is not cleared. This option is useful
         when you wish to pipe the output into a file.

         If -i is passed, then dates of all individual files at the root
         level of a given folder are shown.

Examples:
         $FullScriptName
                Shows the date of the first virscan file it finds in
                the currently installed Engine and Hub folders.

         $FullScriptName -i
                Shows the date of every virus defs file in the currently
                installed Engine and Hub folders. This provides a way to
                see if all the files that have a date stamp in those
                folders are getting updated.

         $FullScriptName /Users/qa/Desktop/microdefs\ folder
                Shows the date of the first virus defs file it finds in
                '/Users/qa/Desktop/microdefs folder' folder.

         $FullScriptName /VIRSCAN1.DAT
                Shows the date of virus defs file VIRSCAN1.DAT, located
                at the root of the boot volume.
" | more -E
   ExitScript 0
}

ShowUsageHelp()
{
   # Usage:     ShowUsageHelp
   # Summary:   Displays script usage message and exits script.
   #
   echo
   echo "For help, type:"
   echo
   echo "   $FullScriptName -h"
   echo
   ExitScript 0
}

# *** Beginning of Commands to Execute ***

# If no arguments were passed to script
if [ $# -eq 0 ] ; then
   # Run script as if it was double-clicked in Finder so that
   # screen will be cleared and quit message will be displayed.
   RunScriptAsStandAlone=true
else
   # Run script in command line mode so that
   # screen won't be cleared and quit message won't be displayed.
   RunScriptAsStandAlone=false
fi
if [ $RunScriptAsStandAlone = true ] ; then
   clear
elif [ "z$1" = z-h -o "z$1" = z-H ] ; then
   ShowUsage
elif [ "z$1" = z-V ] ; then
   echo $Version
   ExitScript 0
fi
if [ "`whoami`" != "root" ]   # If not root user,
   then
   # Run this script again as root
   sudo -p "Please enter your admin password: " "$0" "$@"
   #    If unable to authenticate
   if [ ! $? -eq 0 ] ; then                    
      echo "You entered an invalid password or you are not an admin user. Script aborted."
      ExitScript 1
   fi
   exit 0                     #    Exit so script doesn't run again
fi
while [ "$1" ] ; do
   if [ "z$1" = z-i ] ; then
      ShowIndividualFiles=true
      shift
   elif [ "z$1" = z-c ] ; then
      shift
   else
      break
   fi
done
if [ ! -d "$1" -a ! -f "$1" -a -n "$1" ] ; then
   echo
   echo "Invalid option, file, or folder: '$1'"
   ShowUsageHelp
fi
NumberOfArgumentsPassedToScript=$#
NumberOfArgumentsLeft=$#
echo
while [ $NumberOfArgumentsLeft != 0 ] ; do
   ShowDefsDateForFileOrFolder "$1"
   shift
   NumberOfArgumentsLeft=`expr $NumberOfArgumentsLeft - 1`
done
[ $NumberOfArgumentsPassedToScript -gt 0 ] && ExitScript 0
AntiVirusDirs="$AntiVirusDirOld"
[ ! -d "$AntiVirusDirs" ] && AntiVirusDirs="$AntiVirusDir2"
[ ! -d "$AntiVirusDirs" ] && AntiVirusDirs="$AntiVirusDirs6"
if [ -z "$AntiVirusDirs" ] ; then
   echo "NOTE: Could not find any installed AntiVirus definitions."
   echo
   ExitScript 0
fi
RemainingDirsToCheck=`echo "$AntiVirusDirs" | grep -c .`
IFS='
'
for AntiVirusDir in $AntiVirusDirs ; do
   (( RemainingDirsToCheck-- ))
   cd "$AntiVirusDir" 2>/dev/null
   if [ $? = 0 ] ; then
      echo "AntiVirus definitions in:"
      echo "   \"$AntiVirusDir\""
      echo
      DefsFolderWereFound=false
      for EachInstalledFolder in `ls -d [0-9][0-9][0-9][0-9]* Engine [Hh]ub NewEngine SavedEngine 2>/dev/null | sort -f` ; do
         DefsFolderWereFound=true
         ShowDefsDateForFileOrFolder "$EachInstalledFolder"
      done
      $DefsFolderWereFound || printf "No definitions were found in that folder.\n\n"
   fi
   if [ $RemainingDirsToCheck -gt 0 ] ; then
      echo "-----"
      echo
   fi
done
ExitScript 0

# *** End of Commands to Execute ***
