#!/bin/sh
  Version=2.0.4
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      07/21/2006
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:   See ShowUsage() function.
#
# Summary: See ShowHelp() function.
#
# History: 1.00 - Ported code from GetWCIDdefsDate.command.
#          1.01 - Added check of NewSignatures folder when nothing
#                 is passed to script.
#          1.02 - Broadened installed Signature folder check.
#                 Added -c option.
#          1.03 - Improved how ScriptDir is defined.
#          1.04 - Updated exit code handling after sudo.
#          1.05 - Added modification date to output and removed build
#                 number from combined date string.
#          1.06 - Modified for case-sensitive volume compatibility.
#          2.0.0 - 11/12/2013 - Corey Swertfager:
#                  * Added support for NIS 6.
#          2.0.1 - 06/24/2019 - Corey Swertfager:
#                  * Added support for multiple Silo directories.
#          2.0.2 - 04/03/2020 - Corey Swertfager:
#                  * Changed Author company to NortonLifeLock Inc.
#          2.0.3 - 11/04/2020 - Corey Swertfager:
#                  * Added copyright section.
#          2.0.4 - 12/21/2020 - Corey Swertfager:
#                  * Updated copyright section.

# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
AbbreviatedScriptName="`basename "$0" .command`"
FullScriptName="`basename "$0"`"
IPSDirOld="/Library/Application Support/Symantec/IntrusionPrevention"
IPSDirs6=`ls -d "/Library/Application Support/Symantec/Silo/"*"/Definitions/symcdata/vulnprotectiondefs" 2>/dev/null`
LANG=""
ScriptDir="`dirname "$0"`"
[ -z "`echo "z$ScriptDir" | grep '^z/'`" ] && ScriptDir="`/bin/pwd -P`/$ScriptDir"
ShowIndividualFiles=false
TheFileToCheck=su-cr.md

# *** Function Declarations ***

GetSigsDate()
{
   # Usage:     GetSigsDate $1
   # Argument:  $1 = File from which to get date.
   # Summary:   Gets the date from a file if it is a IPS def file.
   #            Assigns date to SigsDate variable and returns 0 if
   #            a date is found, else assigns NULL to SigsDate and
   #            returns 1.
   #
   SigsDate=""
   SigsMonth=""
   # If $1 is not a file, return 1
   test -f "$1" || return 1
   # If $1 has a filename that begins with a period, return 1
   test "`basename "$1" | awk '{print substr($0,1,1)}'`" = "." && return 1
   ModifiedDateString=`hexdump -C -v -n 16 "$1" | head -n 1`
   ModifiedDateString=`cat "$1" | grep '<Modified>.*</Modified>' | awk -F '>' '{print $2}' | awk -F '<' '{print $1}'`
   # If ModifiedDateString is null, return 1
   [ -z "$ModifiedDateString" ] && return 1
   # If ModifiedDateString is not a number, return 1
   [ -z "`expr "$ModifiedDateString" / 1 2>/dev/null`" ] && return 1
   SigsMonth=`echo "z$ModifiedDateString" | awk '{print substr($0,6,2)}'`
   SigsMonth=`expr $SigsMonth / 1`
   SigsDay=`echo "z$ModifiedDateString" | awk '{print substr($0,8,2)}'`
   SigsYear=`echo "z$ModifiedDateString" | awk '{print substr($0,2,4)}'`
   SigsBuild=`echo "z$ModifiedDateString" | awk '{print substr($0,10)}'`
   SigsBuild=`expr $SigsBuild / 1`
   # If file type was not correct, return 1
   [ -z "$SigsMonth" ] && return 1
   case $SigsMonth in
      1) SigsMonthName=January
         ;;
      2) SigsMonthName=February
         ;;
      3) SigsMonthName=March
         ;;
      4) SigsMonthName=April
         ;;
      5) SigsMonthName=May
         ;;
      6) SigsMonthName=June
         ;;
      7) SigsMonthName=July
         ;;
      8) SigsMonthName=August
         ;;
      9) SigsMonthName=September
         ;;
      10) SigsMonthName=October
         ;;
      11) SigsMonthName=November
         ;;
      12) SigsMonthName=December
         ;;
   esac
   ModificationYear=`ls -dlT "$1" | awk '{print $9}'`
   ModificationTime=`ls -dlT "$1" | awk '{print $8}'`
   ModificationDay=`ls -dlT "$1" | awk '{print $7}'`
   ModificationDay=`expr "$ModificationDay" / 1 2>/dev/null`
   # If ModificationDay is not a number (i.e., fields of ls output are reversed)
   if [ -z "$ModificationDay" ] ; then
      ModificationDay=`ls -dlT "$1" | awk '{print $6}'`
      ModificationMonthText=`ls -dlT "$1" | awk '{print $7}'`
   else
      ModificationMonthText=`ls -dlT "$1" | awk '{print $6}'`
   fi
   SigsDate="$SigsMonthName $SigsDay, $SigsYear - build $SigsBuild (`echo "$ModifiedDateString" | awk '{print substr($0,1,8)}'`), modified $ModificationMonthText $ModificationDay, $ModificationYear $ModificationTime"
   return 0
}

ShowSigsDateForFileOrFolder()
{
   # Usage:     ShowSigsDateForFileOrFolder $1
   # Argument:  $1 = Name of file or folder to check
   # Summary:   Shows IPS signatures date for file or folder passed.
   #
   CurrentDir="`pwd`"
   SigsDate=""
   if [ -d "$1" -a $ShowIndividualFiles = false ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetSigsDate "$FileToCheck"
         # If SigsDate was assigned, break
         test -n "$SigsDate" && break
      done
      if [ -n "$SigsDate" ] ; then
         echo "Date of IPS signatures in folder '$1':"
         echo "   $SigsDate"
      else
         echo "No IPS signatures date was found in any file within folder: '$1'"
      fi
      echo
   elif [ -d "$1" -a $ShowIndividualFiles = true ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetSigsDate "$FileToCheck"
         if [ -n "$SigsDate" ] ; then
            echo "Date of IPS signatures file '$1/$FileToCheck':"
            echo "   $SigsDate"
            echo
         else
            # If filename does not begin with a period (is not invisible)
            if [ "`basename "$FileToCheck" | awk '{print substr($0,1,1)}'`" != "." ] ; then
               echo "No IPS signatures date was found in file: '$1`[ "z$1" != z/ ] && echo /`$FileToCheck'"
               echo
            fi
         fi
      done
   elif [ -f "$1" ] ; then
      GetSigsDate "$1"
      if [ -n "$SigsDate" ] ; then
         echo "Date of IPS signatures file '$1':"
         echo "   $SigsDate"
      else
         echo "No IPS signatures date was found in file: $1"
      fi
      echo
   else
      echo "No such file or folder: '$1'"
      echo
   fi
   cd "$CurrentDir"
}

ShowUsage()
{
   # Usage:     ShowUsage
   # Summary:   Displays script usage message and exits script.
   #
   echo "
Usage:   $FullScriptName [-c] [-i] [files] [folders]

Summary: Shows IPS signatures date of installed IPS signatures if no argument
         is passed or if -c is passed as the only argument, otherwise shows
         IPS signatures dates of all files or folders passed.

         $FullScriptName will report the first date it finds in a
         given folder unless the -i option is passed.

         If -i is passed, then dates of all individual files at the root
         level of a given folder are shown.

         If -c is passed as the first argument, then screen is not cleared;
         if -c is the only argument, then IPS signatures date of installed IPS
         signatures is shown.

Examples:
  $FullScriptName
    Shows the date of the installed signatures.

  $FullScriptName -i
    Shows the date of every IPS signatures file in the installed signatures
    folders.

  $FullScriptName /Users/qa/Desktop/IPS\ signatures\ folder
    Shows the date of the first IPS signatures file it finds in
    '/Users/qa/Desktop/IPS signatures folder' folder.

  $FullScriptName /$TheFileToCheck
    Shows the date of IPS signatures file $TheFileToCheck, located
    at the root of the boot volume.
" | more -E
   exit 0
}

ShowUsageHelp()
{
   # Usage:     ShowUsageHelp
   # Summary:   Displays script usage message and exits script.
   #
   echo
   echo "For help, type:"
   echo
   echo "   $FullScriptName -h"
   echo
   exit 0
}

# *** Beginning of Commands to Execute ***

# If no arguments were passed to script
if [ $# -eq 0 ] ; then
   # Run script as if it was double-clicked in Finder so that
   # screen will be cleared and quit message will be displayed.
   RunScriptAsStandAlone=true
else
   # Run script in command line mode so that
   # screen won't be cleared and quit message won't be displayed.
   RunScriptAsStandAlone=false
fi
if [ $RunScriptAsStandAlone = true ] ; then
   clear
fi
if [ "z$1" = z-h ] ; then
   ShowUsage
elif [ "z$1" = z-V ] ; then
   echo $Version
   exit 0
fi
# If not root user,
if [ "`whoami`" != "root" ] ; then
   # Run this script again as root
   sudo -p "Please enter your admin password: " "$0" "$@"
   ExitCode=$?
   if [ $ExitCode = 1 ] ; then   #    If unable to authenticate,
      echo "You entered an invalid password or you are not an admin user. Script aborted."
   fi
   exit $ExitCode                #    Exit so script doesn't run again
fi
[ "z$1" = z-c ] && shift
if [ "'$1" = "'-i" -o "'$1" = "'-I" ] ; then
   ShowIndividualFiles=true
   shift
elif [ ! -d "$1" -a ! -f "$1" -a -n "$1" ] ; then
   echo
   echo "Invalid option, file, or folder: '$1'"
   ShowUsageHelp
fi
NumberOfArgumentsPassedToScript=$#
NumberOfArgumentsLeft=$#
echo
while [ $NumberOfArgumentsLeft != 0 ] ; do
   ShowSigsDateForFileOrFolder "$1"
   shift
   NumberOfArgumentsLeft=`expr $NumberOfArgumentsLeft - 1`
done
test $NumberOfArgumentsPassedToScript -gt 0 && exit 0
# If NIS 6 or later IPS signatures are installed
if [ "$IPSDirs6" ] ; then
   IPSDirs="$IPSDirs6"
# Else if NIS 5 or earlier IPS signatures are installed
elif [ -d "$IPSDirOld" ] ; then
   IPSDirs="$IPSDirOld"
else
   echo "Could not find any installed Vulnerability Protection (IPS) signatures."
   echo
   exit 3
fi
RemainingDirsToCheck=`echo "$IPSDirs" | grep -c .`
IFS='
'
for IPSDir in $IPSDirs ; do
   (( RemainingDirsToCheck-- ))
   SigFolders=`ls -d "$IPSDir/"*Signatures* "$IPSDir/"2* 2>/dev/null | sort -f`
   SigFolders=`ls -d "$IPSDir/"*Signatures* "$IPSDir/"2* 2>/dev/null | sort -f`
   cd "$IPSDir" 2>/dev/null
   if [ $? = 0 ] ; then
      echo "Vulnerability Protection (IPS) signatures in:"
      echo "   \"$IPSDir\""
      echo
      DefsFolderWereFound=false
      for EachSigFile in $SigFolders ; do
         DefsFolderWereFound=true
         ShowSigsDateForFileOrFolder "`basename "$EachSigFile"`"
      done
      $DefsFolderWereFound || printf "No signatures were found in that folder.\n\n"
   fi
   if [ $RemainingDirsToCheck -gt 0 ] ; then
      echo "-----"
      echo
   fi
done
exit 0

# *** End of Commands to Execute ***
