#!/bin/sh
  Version=2.0.2
# Author:       Corey Swertfager, NortonLifeLock Inc.
# Created:      04/04/2003
# Modified:     12/21/2020
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header start
# //////////////////////
#
# PROPRIETARY / CONFIDENTIAL.
# Use of this product is subject to license terms.
# Copyright (c) 2020 NortonLifeLock Inc.
# All rights reserved.
#
# /////////////////////////////////////////////////////////////////////////////////////////////////
# NortonLifeLock copyright header stop
# //////////////////////
#
# Usage:   GetURLDate.command [-i] [files] [folders]
#
# Summary: Shows date of installed URL list if no argument is passed,
#          otherwise shows date of URL lists in all files or folders
#          passed.
#
#          If -i is passed to script, then dates of each individual file
#          within a given folder are shown. If -i is not passed, then
#          only the first found date within a given folder is shown.
#
# History: 1.00 - Created.
#          1.01 - Now also displays URLLIST version info.
#          1.02 - Now also displays date in yyyymmdd format.
#          1.03 - Added hyphens to date display.
#          1.04 - Modified for OS 10.5 compatibility.
#          1.05 - Added -c option.
#                 Removed ShowVersion function.
#          1.06 - Improved how ScriptDir is defined.
#          1.07 - Added version 3 to error output.
#          1.08 - Modified for case-sensitive volume compatibility.
#          2.0.0 - 04/03/2020 - Corey Swertfager:
#                  * Changed Author company to NortonLifeLock Inc.
#          2.0.1 - 11/04/2020 - Corey Swertfager:
#                  * Added copyright section.
#          2.0.2 - 12/21/2020 - Corey Swertfager:
#                  * Updated copyright section.

# *** Variable Initializations ***

PATH=/bin:/sbin:/usr/bin:/usr/sbin
AbbreviatedScriptName="`basename "$0" .command`"
FullScriptName="`basename "$0"`"
InstalledURLlist=/Library/StartupItems/NortonPrivacyControl/Resources/FURL/FWEB
LANG=""
MinYear=1990
MaxYear=2100
ScriptDir="`dirname "$0"`"
[ -z "`echo "z$ScriptDir" | grep '^z/'`" ] && ScriptDir="`/bin/pwd -P`/$ScriptDir"
ShowIndividualFiles=false

# *** Function Declarations ***

ExitScript()
{
   # Usage:     ExitScript [$1]
   # Argument:  $1 = The value to pass when calling exit. If nothing
   #                 is passed, then 0 is passed to exit.
   # Summary:   Checks to see if RunScriptAsStandAlone variable was
   #            set to true. If so, a message is displayed; otherwise,
   #            no message is displayed. The script is then exited and
   #            passes $1 to exit command.
   #
   if [ -z "$1" ] ; then
      exit 0
   else
      exit $1
   fi
}

GetURLDate()
{
   # Usage:     GetURLDate $1
   # Argument:  $1 = File from which to get date.
   # Summary:   Gets the date from a file if it is a URL list file.
   #            Assigns date to URLDate variable and returns 0 if
   #            a date is found, else assigns NULL to URLDate and
   #            returns 1.
   #
   URLDate=""
   # If $1 is not a file, return 1
   [ ! -f "$1" ] && return 1
   # If $1 has a filename that begins with a period, return 1
   [ "`basename "$1" | awk '{print substr($0,1,1)}'`" = "." ] && return 1
   # If incremental updater
   if [ "z`hexdump -C -v -n 9 "$1" | awk '{print $11}'`"  = "z|increment|" ] ; then
      URLDay=`hexdump -C -v -s 90 -n 2 "$1" | tr -d "|" | awk '{print $4}'`
      URLMonth=`hexdump -C -v -s 93 -n 3 "$1" | tr -d "|" | awk '{print $5}'`
      URLYear=`hexdump -C -v -s 97 -n 4 "$1" | tr -d "|" | awk '{print $6}'`
      URLVersion=`head -n 1 "$1"`
   else
      URLDay=`hexdump -C -v -s 18 -n 2 "$1" | tr -d "|" | awk '{print $4}'`
      URLMonth=`hexdump -C -v -s 21 -n 3 "$1" | tr -d "|" | awk '{print $5}'`
      URLYear=`hexdump -C -v -s 25 -n 4 "$1" | tr -d "|" | awk '{print $6}'`
      URLVersion=`head -n 1 "$1" | awk '{print $3}'`
   fi
   URLYear=`expr $URLYear / 1 2>/dev/null`
   [ -z "$URLYear" ] && return 1   # If year is not a number, return 1
   [ $URLYear -lt $MinYear -o $URLYear -gt $MaxYear ] && return 1   # If year is not valid, return 1
   URLDay=`expr $URLDay / 1 2>/dev/null`
   [ -z "$URLDay" ] && return 1   # If day is not a number, return 1
   [ $URLDay -lt 1 -o $URLDay -gt 31 ] && return 1   # If day is not valid, return 1
   case "$URLMonth" in
      Jan) URLMonthName=January
         URLMonthNumber=01
         ;;
      Feb) URLMonthName=February
         URLMonthNumber=02
         ;;
      Mar) URLMonthName=March
         URLMonthNumber=03
         ;;
      Apr) URLMonthName=April
         URLMonthNumber=04
         ;;
      May) URLMonthName=May
         URLMonthNumber=05
         ;;
      Jun) URLMonthName=June
         URLMonthNumber=06
         ;;
      Jul) URLMonthName=July
         URLMonthNumber=07
         ;;
      Aug) URLMonthName=August
         URLMonthNumber=08
         ;;
      Sep) URLMonthName=September
         URLMonthNumber=09
         ;;
      Oct) URLMonthName=October
         URLMonthNumber=10
         ;;
      Nov) URLMonthName=November
         URLMonthNumber=11
         ;;
      Dec) URLMonthName=December
         URLMonthNumber=12
         ;;
      *) return 1
         ;;
   esac
   URLDayNumber=$URLDay
   [ $URLDayNumber -lt 10 ] && URLDayNumber="0$URLDayNumber"
   URLDateText="$URLMonthName $URLDay, $URLYear"
   URLDate="$URLDateText  -  $URLYear-$URLMonthNumber-$URLDayNumber"
   return 0
}

ShowURLDateForFileOrFolder()
{
   # Usage:     ShowURLDateForFileOrFolder $1
   # Argument:  $1 = Name of file or folder to check
   # Summary:   Shows URL list date for file or folder passed.
   #
   CurrentDir="`pwd`"
   URLDate=""
   if [ -d "$1" -a $ShowIndividualFiles = false ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetURLDate "$FileToCheck"
         # If URLDate was assigned, break
         [ -n "$URLDate" ] && break
      done
      if [ -n "$URLDate" ] ; then
         echo "Date of URL list in folder '$1':"
         echo "   $URLDate  (version: $URLVersion)"
      else
         echo "No URL list date was found in any file within folder: $1"
      fi
      echo
   elif [ -d "$1" -a $ShowIndividualFiles = true ] ; then
      cd "$1"
      IFS='
'
      for FileToCheck in `ls` ; do
         GetURLDate "$FileToCheck"
         if [ -n "$URLDate" ] ; then
            echo "Date of URL list '$1/$FileToCheck':"
            echo "   $URLDate  (version: $URLVersion)"
            echo
         else
            # If filename does not begin with a period (is not invisible)
            if [ "`basename "$FileToCheck" | awk '{print substr($0,1,1)}'`" != "." ] ; then
               echo "No URL list date was found in file: $1/$FileToCheck"
               echo
            fi
         fi
      done
   elif [ -f "$1" ] ; then
      GetURLDate "$1"
      if [ -n "$URLDate" ] ; then
         if [ "$1" = "$InstalledURLlist" ] ; then
            echo "Date of installed URL list:"
         else
            echo "Date of URL list '$1':"
         fi
         echo "   $URLDate  (version: $URLVersion)"
      else
         echo "No URL list date was found in file: $1"
      fi
      echo
   else
      echo "No such file or folder: '$1'"
      echo
   fi
   cd "$CurrentDir"
}

ShowUsage()
{
   # Usage:     ShowUsage
   # Summary:   Displays script usage message and exits script.
   #
   clear
   echo "
Usage:   $FullScriptName [-c] [-i] [files] [folders]

Summary: Shows date of installed URL list if no argument is passed,
         otherwise shows date of URL lists in all files or folders
         passed.

         If -c is passed, then screen is not cleared. This option is useful
         when you wish to pipe the output into a file.

         $FullScriptName will report the first date it finds in a given
         folder unless the -i option is passed.

         If -i is passed, then dates of all individual files at the root
         level of a given folder are shown.

Examples:
         $FullScriptName
                Shows the date of the installed URL list.

         $FullScriptName /Users/qa/Desktop/SymantecURLs.osx\ folder
                Shows the date of the first URL list file it finds in
                \"/Users/qa/Desktop/SymantecURLs.osx folder\" folder.

         $FullScriptName -i \"test folder\"
                Shows the date of every URL list file in the folder
                \"test folder\" in the current directory.

         $FullScriptName /URLLIST.LST
                Shows the date of URL list file URLLIST.LST, located
                at the root of the boot volume.
" | more -E
   ExitScript 0
}

ShowUsageHelp()
{
   # Usage:     ShowUsageHelp
   # Summary:   Displays script usage message and exits script.
   #
   echo
   echo "For help, type:"
   echo
   echo "   $FullScriptName -h"
   echo
   ExitScript 0
}

# *** Beginning of Commands to Execute ***

# If no arguments were passed to script
if [ $# -eq 0 ] ; then
   # Run script as if it was double-clicked in Finder so that
   # screen will be cleared and quit message will be displayed.
   RunScriptAsStandAlone=true
else
   # Run script in command line mode so that
   # screen won't be cleared and quit message won't be displayed.
   RunScriptAsStandAlone=false
fi
if [ $RunScriptAsStandAlone = true ] ; then
   clear
elif [ "z$1" = z-h -o "z$1" = z-H ] ; then
   ShowUsage
elif [ "z$1" = z-V ] ; then
   echo $Version
   ExitScript 0
fi
if [ "`whoami`" != "root" ] ; then
   # Run this script again as root
   sudo -p "Please enter your admin password: " "$0" "$@"
   #    If unable to authenticate
   if [ ! $? -eq 0 ] ; then                    
      echo "You entered an invalid password or you are not an admin user. Script aborted."
      ExitScript 1
   fi
   exit 0                     #    Exit so script doesn't run again
fi
while [ "$1" ] ; do
   if [ "z$1" = z-i ] ; then
      ShowIndividualFiles=true
      shift
   elif [ "z$1" = z-c ] ; then
      shift
   else
      break
   fi
done
if [ ! -d "$1" -a ! -f "$1" -a -n "$1" ] ; then
   echo
   echo "Invalid option, file, or folder: '$1'"
   ShowUsageHelp
fi
NumberOfArgumentsPassedToScript=$#
NumberOfArgumentsLeft=$#
echo
while [ $NumberOfArgumentsLeft != 0 ] ; do
   ShowURLDateForFileOrFolder "$1"
   shift
   NumberOfArgumentsLeft=`expr $NumberOfArgumentsLeft - 1`
done
[ $NumberOfArgumentsPassedToScript -gt 0 ] && ExitScript 0
if [ -f "$InstalledURLlist" ] ; then
   ShowURLDateForFileOrFolder "$InstalledURLlist"
else
   echo "Could not find '$InstalledURLlist'."
   echo "Perhaps Norton Internet Security 3 is not installed."
   echo
fi
ExitScript 0

# *** End of Commands to Execute ***
